from flask_wtf import FlaskForm
from wtforms import SubmitField, HiddenField, StringField
from wtforms.validators import InputRequired, Length, Regexp


class AddTeacher(FlaskForm):
    # id used only by update/edit
    id_field = HiddenField()
    firstname = StringField('Voornaam', [InputRequired(),
        Regexp(r'^[A-Za-z\s\-\']+$', message="Invalid sock name"),
        Length(min=2, max=25, message="Invalid sock name length")
        ])
    lastname = StringField('Achternaam', [InputRequired(),
        Regexp(r'^[A-Za-z\s\-\']+$', message="Invalid sock name"),
        Length(min=2, max=25, message="Invalid sock name length")
        ])

    # updated - date - handled in the route
    updated = HiddenField()
    submit = SubmitField('Add/Update Record')


class EditTeacher(FlaskForm):
    # id used only by update/edit
    id_field = HiddenField()
    lastname = StringField('Achternaam', [InputRequired(),
        Regexp(r'^[A-Za-z\s\-\']+$', message="Invalid sock name"),
        Length(min=2, max=25, message="Invalid sock name length")
        ])
    firstname = StringField('Voornaam', [InputRequired(),
        Regexp(r'^[A-Za-z\s\-\']+$', message="Invalid sock name"),
        Length(min=2, max=25, message="Invalid sock name length")
        ])
    # updated - date - handled in the route
    updated = HiddenField()
    submit = SubmitField('opslaan')



from flask import render_template, Blueprint, request, flash
from sqlalchemy.orm import Query

from src.classes.base import RearrangeDate
from src.classes.family import AddFamily, EditFamily
from src.classes.database import Child, Family, sessionSetup
from src.modules.children import find_related_children
from src.modules.common import stringdate, find_parents
from src.modules.families import find_family, find_families, provision_edit_family

session = sessionSetup()
rearrange_date = RearrangeDate()

# Blueprint Configuration
families_bp = Blueprint(
    'families_bp', __name__,
    template_folder='templates'
)


@families_bp.route('/index_families')
def index_families():
    return render_template('index_families.html',
                           Families=find_families(fid=None),
                           _PageTitle='Familie Overzicht')


@families_bp.route('/details_family/<fid>')
def details_family(fid):
    try:
        # _family = find_families(fid=fid)
        _family = provision_edit_family(fid)
        _children = find_related_children(fid)
        return render_template('details_family.html',
                               Family=_family,
                               Children=_children,
                               _PageTitle='Gezins details')
    except Exception as e:
        # e holds description of the error
        error_text = "<p>The error:<br>" + str(e) + "</p>"
        hed = '<h1>Something is broken.</h1>'
        return hed + error_text


@families_bp.route('/edit_family/<fid>')
def edit_family(fid):
    _familyToEdit = provision_edit_family(fid)
    return render_template('edit_family.html',
                           familyToEdit=_familyToEdit,
                           form1=EditFamily(),
                           _PageTitle='Gezin wijzigen')


@families_bp.route('/add_family', methods=['GET', 'POST'])
def add_family():
    _form1 = AddFamily()
    if _form1.validate_on_submit():
        id_field = request.form['id_field']
        parent1_id = request.form['parent1_id']
        parent2_id = request.form['parent2_id']
        divorced = request.form['divorced']
        # the data to be inserted into Sock model - the table, socks
        record = Family(id_field, parent1_id, parent2_id, divorced)
        # Flask-SQLAlchemy magic adds record to database
        session.add(record)
        session.commit()
        # create a message to send to the template
        _message = f"Gezin {id_field} is aangemaakt."
        return render_template('add_family.html',
                               message=_message,
                               _PageTitle='Gezin toevoegen')
    else:
        # show validaton errors
        # see https://pythonprogramming.net/flash-flask-tutorial/
        for field, errors in _form1.errors.items():
            for error in errors:
                flash("Error in {}: {}".format(
                    getattr(_form1, field).label.text,
                    error
                ), 'error')
        return render_template('add_family.html',
                               form1=_form1,
                               _PageTitle='Gezin toevoegen')


@families_bp.route('/remove_family/<fid>')
def remove_family(fid):
    try:
        _child_to_delete = Query(Child).with_session(session).filter(Child.family_id == fid).delete()
    except:
        pass
    _family_to_delete = Query(Family).with_session(session).filter(Family.id == fid).delete()
    session.commit()
    message = f"De gegevens zijn verwijderd."
    return render_template('remove_family_result.html',
                           message=message,
                           _PageTitle='Gezin verwijderen')


@families_bp.route('/edit_family_result', methods=['POST'])
def edit_family_result():
    fid = request.form['id_field']
    _familyToEdit = provision_edit_family(fid)
    _form1 = EditFamily()
    if _form1.validate_on_submit():
        parent1_id = _form1.parent1_id.data
        parent2_id = _form1.parent2_id.data
        divorced = _form1.divorced.data
        _family_to_edit = Query(Family).with_session(session).filter(Family.id == fid).update(dict(
            parent1_id=parent1_id,
            parent2_id=parent2_id,
            divorced=divorced))
        session.commit()
        message = f"De gegevens voor {_familyToEdit['parent1']['lastname']} zijn bijgewerkt."
        return render_template('result.html',
                               message=message,
                               redirect=f'details_family/{fid}',
                               _PageTitle='Resultaat')
    else:
        # show validaton errors
        _familyToEdit["id"] = fid
        # see https://pythonprogramming.net/flash-flask-tutorial/
        for field, errors in _form1.errors.items():
            for error in errors:
                flash("Error in {}: {}".format(
                    getattr(_form1, field).label.text,
                    error
                ), 'error')
        return render_template('edit_family.html',
                               form1=_form1,
                               familyToEdit=_familyToEdit,
                               redirect=f'details_family/{fid}',
                               choice='edit',
                               _PageTitle='Resultaat')



# Hier komt alles samen
from flask import Flask, render_template
from flask_assets import Environment  # Import `Environment`
from flask_bootstrap import Bootstrap

import frontend.children.routes
import frontend.families.routes
import frontend.teachers.routes
import frontend.classrooms.routes
from src.classes.database import create_database, sessionSetup
from src.modules.config import load_config

config = load_config()

def create_app():
    """Create Flask application."""
    #https://hackersandslackers.com/configure-flask-applications
    _app = Flask(__name__, instance_relative_config=False)
    _app.config['SECRET_KEY'] = config["flask"]["secretKey"]
    assets = Environment()  # Create an assets environment
    assets.init_app(_app)  # Initialize Flask-Assets

    with _app.app_context():
        # Import parts of our application
        from frontend import routes
        from frontend.teachers import routes
        from frontend.classrooms import routes
        from frontend.children import routes
        from frontend.families import routes
        from frontend.parents import routes

        # Register Blueprints
        _app.register_blueprint(frontend.routes.templates_bp)
        _app.register_blueprint(frontend.teachers.routes.teachers_bp)
        _app.register_blueprint(frontend.classrooms.routes.classroom_bp)
        _app.register_blueprint(frontend.children.routes.children_bp)
        _app.register_blueprint(frontend.families.routes.families_bp)
        _app.register_blueprint(frontend.parents.routes.parents_bp)

        return _app


app = create_app()
Bootstrap(app)
databasefile = config["database"]["path"] + config["database"]["name"]
session = sessionSetup()


if __name__ == '__main__':
    from os.path import exists
    import webbrowser

    file_exists = exists(databasefile)
    if not file_exists:
        print(f'Database aanmaken..')
        create_database(databasefile=databasefile)
    # webbrowser.open_new_tab('http://localhost:5000')
    app.run(debug=True)



from flask_wtf import FlaskForm
from sqlalchemy.orm import Query
from wtforms import SubmitField, SelectField, HiddenField, StringField, DateField
from wtforms.validators import InputRequired, Length, Regexp

from src.classes.database import Teacher, sessionSetup

session = sessionSetup()


class AddClass(FlaskForm):
    # id used only by update/edit
    id_field = HiddenField()
    class_name = StringField('Naam van de Klas', [InputRequired(),
                                                  Regexp(r'^[A-Za-z\s\-\']+$', message="Invalid sock name"),
                                                  Length(min=2, max=25, message="Invalid sock name length")
                                                  ])
    teacher = SelectField(u'Docent', [InputRequired()], coerce=int, choices=Query([
        Teacher.id,
        Teacher.firstname + ' ' + Teacher.lastname]).with_session(session))
    start_date = DateField('Datum van starten', [InputRequired()], format='%d-%m-%Y',
                           render_kw={"placeholder": "dd-mm-jjjj"})
    end_date = DateField('Datum van stoppen', [InputRequired()], format='%d-%m-%Y',
                         render_kw={"placeholder": "dd-mm-jjjj"})

    # updated - date - handled in the route
    updated = HiddenField()
    submit = SubmitField('Add/Update Record')


class EditClass(FlaskForm):
    # id used only by update/edit
    id_field = HiddenField()
    class_name = StringField('Naam van de Klas', [InputRequired(),
                                                  Regexp(r'^[A-Za-z\s\-\']+$', message="Invalid sock name"),
                                                  Length(min=2, max=25, message="Invalid sock name length")
                                                  ])
    teacher = SelectField(u'Docent', [InputRequired()], coerce=int, choices=Query([
        Teacher.id,
        Teacher.firstname + ' ' + Teacher.lastname]).with_session(session))
    start_date = DateField('Datum van starten', [InputRequired()], format='%d-%m-%Y',
                           render_kw={"placeholder": "dd-mm-jjjj"})
    end_date = DateField('Datum van stoppen', [InputRequired()], format='%d-%m-%Y',
                         render_kw={"placeholder": "dd-mm-jjjj"})

    # updated - date - handled in the route
    updated = HiddenField()
    submit = SubmitField('Add/Update Record')



from flask import Blueprint, request, flash
from flask import render_template
from sqlalchemy.orm import Query

from src.modules.common import calculate_age, find_teacher, stringdate
from src.classes.base import RearrangeDate
from src.classes.classroom import AddClass, EditClass
from src.classes.database import Class, Child, sessionSetup, Teacher

session = sessionSetup()
rearrange_date = RearrangeDate()

# Blueprint Configuration
classroom_bp = Blueprint(
    'classroom_bp', __name__,
    template_folder='templates'
)


@classroom_bp.route('/index_classes')
def index_classrooms():
    # get a list of unique values in the style column
    _classes_from_database = Query([
        Class.id,
        Class.class_name,
        Class.teacher,
        Class.start_date,
        Class.end_date
    ]).with_session(session)
    _classes = []
    for _class in _classes_from_database:
        _classes.append({
            'id': _class[0],
            'class_name': _class[1] + ' ' + _class[3][0:4],
            'teacher': find_teacher(_class[2]),
            'start_date': _class[3],
            'end_date': _class[4],
        })
    return render_template('index_classrooms.html',
                           Classes=_classes,
                           _PageTitle='Klassen Overzicht')


@classroom_bp.route('/add_class', methods=['GET', 'POST'])
def add_classroom():
    _form1 = AddClass()
    if _form1.validate_on_submit():
        id_field = request.form['id_field']
        class_name = request.form['class_name']
        teacher = request.form['teacher']
        start_date = request.form['start_date']
        start_date = rearrange_date.to_order(start_date)[0]  # Rearrange date for correct filing
        end_date = request.form['end_date']
        end_date = rearrange_date.to_order(end_date)[0]  # Rearrange date for correct filing
        record = Class(id_field, class_name, teacher, start_date, end_date)
        # Flask-SQLAlchemy magic adds record to database

        session.add(record)
        session.commit()
        # create a message to send to the template
        _message = f"Klas '{class_name}' is aangemaakt."
        return render_template('add_classroom.html',
                               message=_message,
                               _PageTitle='Klas toevoegen')
    else:
        # show validaton errors
        # see https://pythonprogramming.net/flash-flask-tutorial/
        for field, errors in _form1.errors.items():
            for error in errors:
                flash("Error in {}: {}".format(
                    getattr(_form1, field).label.text,
                    error
                ), 'error')
        return render_template('add_classroom.html',
                               form1=_form1,
                               _PageTitle='Klas toevoegen')


@classroom_bp.route('/details_class/<cid>')
# TODO, kolom aanmaken voor wanneer kind de klas verlaat. ergens bij pasen?

def details_classroom(cid):
    try:
        _class_from_database = Query([
            Class.class_name,
            Class.teacher,
            Class.id,
            Class.start_date,
            Class.end_date
        ]).with_session(session).filter(Class.id == cid).all()
        _class = []
        for _class_from_database in _class_from_database:
            _class.append({
                'class_name': _class_from_database.class_name,
                'teacher': _class_from_database.teacher,
                'start_date': _class_from_database.start_date,
                'end_date': _class_from_database.end_date,
                'id': _class_from_database.id
            })
        _teacher_for_class = Query([
            Teacher.firstname,
            Teacher.lastname
        ]).with_session(session).filter(Teacher.id == _class_from_database.teacher).all()
        _children_in_class = Query([
            Child.firstname,
            Child.lastname,
            Child.date_of_birth,
            Child.id
        ]).with_session(session).filter(Child.class_id == _class_from_database.id).all()
        _children = []
        for _c in _children_in_class:
            _children.append({
                "firstname": _c[0],
                "lastname": _c[1],
                "age": calculate_age(_c[2]),
                "id": _c[3]
            })
        return render_template('details_classroom.html',
                               Class=_class,
                               Teacher=_teacher_for_class,
                               Children=_children,
                               _PageTitle='Klas details')
    except Exception as e:
        # e holds description of the error
        error_text = "<p>The error:<br>" + str(e) + "</p>"
        hed = '<h1>Something is broken.</h1>'
        return hed + error_text


@classroom_bp.route('/edit_class/<cid>')
def edit_classroom(cid):
    _class_from_database = Query([
        Class.class_name,
        Class.teacher,
        Class.start_date,
        Class.end_date,
        Class.id
    ]).with_session(session).filter(Class.id == cid).first()
    # two forms in this template
    _classToEdit = {
        'id': _class_from_database.id,
        'class_name': _class_from_database.class_name,
        'teacher': _class_from_database.teacher,
        'start_date': rearrange_date.to_list(_class_from_database.start_date)[0],
        'end_date': rearrange_date.to_list(_class_from_database.end_date)[0]}
    return render_template('edit_classroom.html',
                           classToEdit=_classToEdit,
                           form1=EditClass(),
                           _PageTitle='Klas wijzigen')


@classroom_bp.route('/remove_class/<cid>')
def remove_classroom(cid):
    _class_to_remove = Query(Class).with_session(session).filter(Class.id == cid).delete()
    session.commit()
    message = f"De gegevens zijn verwijderd."
    return render_template('remove_class_result.html',
                           message=message,
                           _PageTitle='Klas verwijderen')


@classroom_bp.route('/edit_class_result', methods=['POST'])
def edit_classroom_result():
    cid = request.form['id_field']
    _classToEdit = {"id": cid,
                      "class_name": request.form['class_name'],
                      "teacher": request.form['teacher'],
                      "start_date": rearrange_date.to_order(request.form['start_date'])[0],
                      "end_date": rearrange_date.to_order(request.form['end_date'])[0],
                      "updated": stringdate()}
    _form1 = EditClass()
    if _form1.validate_on_submit():
        class_name = _form1.class_name.data
        teacher = _form1.teacher.data
        start_date = _form1.start_date.data
        end_date = _form1.end_date.data
        _class_to_edit = Query(Class).with_session(session).filter(Class.id == cid).update(dict(
            class_name=class_name,
            teacher=teacher,
            start_date=start_date,
            end_date=end_date))
        session.commit()
        message = f"De gegevens voor {_classToEdit['class_name']} zijn bijgewerkt."
        return render_template('result.html',
                               message=message,
                               redirect=f'details_class/{cid}',
                               _PageTitle='Resultaat')
    else:
        # show validaton errors
        _classToEdit["id"] = cid
        # see https://pythonprogramming.net/flash-flask-tutorial/
        for field, errors in _form1.errors.items():
            for error in errors:
                flash("Error in {}: {}".format(
                    getattr(_form1, field).label.text,
                    error
                ), 'error')
        return render_template('edit_classroom.html',
                               form1=_form1,
                               classToEdit=_classToEdit,
                               redirect=f'details_class/{cid}',
                               choice='edit',
                               _PageTitle='Resultaat')



from datetime import date

from sqlalchemy.orm import Query

from src.classes.child import rearrange_date
from src.classes.database import Child, sessionSetup, Teacher, Class, Family, Parent
from src.modules.classrooms import get_classroom
from src.modules.common import find_parents, calculate_age, find_class

session = sessionSetup()


def find_related_children(fid):
    query_entities = [
        Child.id,
        Child.family_id,
        Child.firstname,
        Child.lastname,
        Child.date_of_birth,
        Child.class_id
    ]
    _queried_child = Query(query_entities).filter(Child.family_id == fid).with_session(session)
    _classes = Query(Class).with_entities(Class.id, Class.class_name, Class.start_date).with_session(session)
    children = []
    for c in _queried_child:
        children.append({
            'id': c['id'],
            'family_id': c['family_id'],
            'firstname': c['firstname'],
            'lastname': c['lastname'],
            'age': calculate_age(c['date_of_birth']),
            'classroom': get_classroom(c['class_id'])
        })
    # calculate age
    # find class
    return children



from datetime import date

from sqlalchemy.orm import Query

from src.classes.database import Child, sessionSetup, Teacher, Class, Family, Parent
from src.modules.common import find_parents

session = sessionSetup()


def find_families(fid):
    query_entities = [
        Family.id,
        Family.parent1_id,
        Family.parent2_id,
        Family.divorced,
    ]
    if fid is None:
        _families_from_database = Query(query_entities).with_session(session)
    else:
        return Query(query_entities).filter(Family.id == fid).with_session(session)
    _families = []
    for _family in _families_from_database:
        _families.append({
            'id': _family[0],
            'parent1_id': _family[1],
            'parent1': find_parents(_family[1]),
            'parent2_id': _family[2],
            'parent2': find_parents(_family[2]),
            'divorced': _family[3]
        })
    return _families


def provision_edit_family(fid):
    _family = find_families(fid)[0]
    _parent1 = find_parents(_family[1])
    _parent2 = find_parents(_family[2])
    _family_to_edit = {'id': _family[0],
                       'parent1': {
                           'id': _parent1[0],
                           'firstname': _parent1[1],
                           'lastname': _parent1[2],
                           'address': _parent1[3],
                           'city': _parent1[4],
                           'zipcode': _parent1[5],
                           'phone': _parent1[6],
                           'email': _parent1[7]
                       },
                       'parent2': {
                           'id': _parent2[0],
                           'firstname': _parent2[1],
                           'lastname': _parent2[2],
                           'address': _parent2[3],
                           'city': _parent2[4],
                           'zipcode': _parent2[5],
                           'phone': _parent2[6],
                           'email': _parent2[7]
                       },
                       'divorced': _family[3]
                       }
    return _family_to_edit


def find_family():
    _families_from_database = find_families('')
    # family table
    # parents opzoeken
    # tuple aanmaken voor ouders met namen enzo



from flask import Blueprint, request, flash
from flask import render_template
from sqlalchemy.orm import Query

from src.modules.common import find_teacher
from src.classes.teacher import AddTeacher, EditTeacher
from src.classes.database import sessionSetup, Teacher

session = sessionSetup()

# Blueprint Configuration
teachers_bp = Blueprint(
    'teachers_bp', __name__,
    template_folder='templates'
)


@teachers_bp.route('/index_teacher')
def index_teachers():
    # get a list of unique values in the style column
    _teachers_from_database = Query([
        Teacher.id,
        Teacher.firstname,
        Teacher.lastname,
    ]).with_session(session)
    _teachers = []
    for _teacher in _teachers_from_database:
        _teachers.append({
            'id': _teacher[0],
            'firstname': _teacher[1],
            'lastname': _teacher[2],
        })
    return render_template('index_teachers.html',
                           Teachers=_teachers,
                           _PageTitle='Docenten Overzicht')


@teachers_bp.route('/add_teacher', methods=['GET', 'POST'])
def add_teacher():
    _form1 = AddTeacher()
    if _form1.validate_on_submit():
        id_field = request.form['id_field']
        firstname = request.form['firstname']
        lastname = request.form['lastname']
        # the data to be inserted into Sock model - the table, socks
        record = Teacher(id_field, firstname, lastname)
        # Flask-SQLAlchemy magic adds record to database
        session.add(record)
        session.commit()
        # create a message to send to the template
        _message = f"Leraar '{firstname} {lastname}' is aangemaakt."
        return render_template('add_teacher.html',
                               message=_message,
                               _PageTitle='Docent toevoegen')
    else:
        # show validaton errors
        # see https://pythonprogramming.net/flash-flask-tutorial/
        for field, errors in _form1.errors.items():
            for error in errors:
                flash("Error in {}: {}".format(
                    getattr(_form1, field).label.text,
                    error
                ), 'error')
        return render_template('add_teacher.html',
                               form1=_form1,
                               _PageTitle='Docent toevoegen')


@teachers_bp.route('/details_teacher/<tid>')
def details_teacher(tid):
    try:
        _teacher_from_database = Query([
            Teacher.firstname,
            Teacher.lastname,
            Teacher.id,
        ]).with_session(session).filter(Teacher.id == tid).all()
        _teacher = []
        for _teacher_from_database in _teacher_from_database:
            _teacher.append({
                'firstname': _teacher_from_database.firstname,
                'lastname': _teacher_from_database.lastname,
                'id': _teacher_from_database.id
            })
        return render_template('details_teacher.html',
                               Teacher=_teacher,
                               _PageTitle='Docent details')
    except Exception as e:
        # e holds description of the error
        error_text = "<p>The error:<br>" + str(e) + "</p>"
        hed = '<h1>Something is broken.</h1>'
        return hed + error_text


@teachers_bp.route('/edit_teacher/<tid>')
def edit_teacher(tid):
    _teacher_from_database = Query([
        Teacher.firstname,
        Teacher.lastname,
        Teacher.id,
    ]).with_session(session).filter(Teacher.id == tid).with_session(session).first()
    # two forms in this template
    _teacherToEdit = {'firstname': _teacher_from_database.firstname,
                      'lastname': _teacher_from_database.lastname,
                      'id': _teacher_from_database.id}
    return render_template('edit_teacher.html',
                           teacherToEdit=_teacherToEdit,
                           form1=EditTeacher(),
                           _PageTitle='Docent wijzigen')


@teachers_bp.route('/edit_teacher_result', methods=['POST'])
def edit_teacher_result():
    tid = request.form['id_field']
    _teacher_from_database = find_teacher(tid)
    #     Query([
    #     Teacher.firstname,
    #     Teacher.lastname,
    #     Teacher.id,
    # ]).with_session(session).filter(Teacher.id == tid).with_session(session).first()
    # two forms in this template
    _teacherToEdit = {'id': tid,
                      'firstname': _teacher_from_database.firstname,
                      'lastname': _teacher_from_database.lastname}
    _form1 = EditTeacher()
    if _form1.validate_on_submit():
        firstname = _form1.firstname.data
        lastname = _form1.lastname.data
        _teacher_to_edit = Query(Teacher).with_session(session).filter(Teacher.id == tid).update(dict(
            firstname=firstname,
            lastname=lastname))
        session.commit()
        message = f"De gegevens voor {_teacherToEdit['firstname']} zijn bijgewerkt."
        return render_template('result.html',
                               message=message,
                               redirect=f'details_teacher/{tid}',
                               _PageTitle='Resultaat')
    else:
        _teacherToEdit['id'] = tid
        for field, errors in _form1.errors.items():
            for error in errors:
                flash("Error in {}: {}".format(
                    getattr(_form1, field).label.text,
                    error
                ), 'error')
        return render_template('edit_teacher.html',
                               form1=_form1,
                               teacherToEdit=_teacherToEdit,
                               choice='edit',
                               redirect=f'details_teacher/{tid}',
                               _PageTitle='Resultaat')


@teachers_bp.route('/remove_teacher/<tid>')
def remove_teacher(tid):
    _teacher_to_remove = Query(Teacher).with_session(session).filter(Teacher.id == tid).delete()
    session.commit()
    message = f"De gegevens zijn verwijderd."
    return render_template('remove_teacher_result.html',
                           message=message,
                           _PageTitle='Docent verwijderen')



from flask_wtf import FlaskForm
from sqlalchemy.orm import Query
from wtforms import SubmitField, HiddenField, StringField, BooleanField, SelectField
from wtforms.validators import InputRequired, Length, Regexp
from flask_sqlalchemy import SQLAlchemy
from src.modules.common import calculate_age
from src.classes.database import Child, sessionSetup, Class, Parent

db = SQLAlchemy()
session = sessionSetup()


class AddFamily(FlaskForm):
    # id used only by update/edit
    id_field = HiddenField()
    parent1_id = SelectField(u'Ouders', [InputRequired()],
                             coerce=int,
                             choices=Query([Parent.id, Parent.firstname + ' ' + Parent.lastname]).with_session(session))
    parent2_id = SelectField(u'Ouders', [InputRequired()],
                             coerce=int,
                             choices=Query([Parent.id, Parent.firstname + ' ' + Parent.lastname]).with_session(session))
    divorced = SelectField(u'Gescheiden',
                           coerce=int,
                           choices=[(0, 'Nee'), (1, 'Ja')])
    # updated - date - handled in the route
    updated = HiddenField()
    submit = SubmitField('Add/Update Record')


class EditFamily(FlaskForm):
    # id used only by update/edit
    id_field = HiddenField()
    parent1_id = SelectField(u'Ouders', [InputRequired()],
                             coerce=int,
                             choices=Query([Parent.id, Parent.firstname + ' ' + Parent.lastname]).with_session(session))
    parent2_id = SelectField(u'Ouders', [InputRequired()],
                             coerce=int,
                             choices=Query([Parent.id, Parent.firstname + ' ' + Parent.lastname]).with_session(session))
    divorced = SelectField(u'Gescheiden',
                           coerce=int,
                           choices=[(0, 'Nee'), (1, 'Ja')])
    # updated - date - handled in the route
    updated = HiddenField()
    submit = SubmitField('opslaan')


def find_children(fid):
    _children_from_database = Query(
        [Child.id, Child.firstname, Child.date_of_birth, Child.family, Child.class_id]).filter(
        Child.family == fid).with_session(session)
    _child = []
    for _c in _children_from_database:
        if _c[4] is None:
            _class_name = 'N/A'
        else:
            _class_name = Query(Class.class_name).filter(Class.id == _c[4]).with_session(session)[0][0]
        _child.append({
            'id': _c[0],
            'firstname': _c[1],
            'date_of_birth': _c[2],
            'child_age': calculate_age(_c[2]),
            'family': _c[3],
            'class': _class_name
        })
    return _child



from flask import Blueprint, request, flash
from flask import render_template
from src.modules.common import find_waiting_children

from src.classes.database import sessionSetup

session = sessionSetup()

# Blueprint Configuration
templates_bp = Blueprint(
    'templates_bp', __name__,
    template_folder='templates'
)


@templates_bp.route('/')
def index():
    _waiting_children = find_waiting_children()
    return render_template('index.html',
                           children=_waiting_children,
                           _PageTitle='Wachtlijst')



from flask import render_template, Blueprint, request, flash
from sqlalchemy.orm import Query

from src.modules.common import stringdate, calculate_age, calculate_startyear, find_class, find_teacher, \
    query_class_by_year, find_child, get_appropriate_class
from src.classes.base import RearrangeDate
from src.classes.child import AddChild, EditChild
from src.classes.database import Child, Family, sessionSetup
from src.modules.families import provision_edit_family

session = sessionSetup()
rearrange_date = RearrangeDate()


# Blueprint Configuration
children_bp = Blueprint(
    'children_bp', __name__,
    template_folder='templates'
)


@children_bp.route('/index_children')
def index_children():
    # get a list of unique values in the style column
    _children_from_database = Query([
        Child.firstname,
        Child.lastname,
        Child.date_of_registration,
        Child.date_of_birth,
        Child.id
    ]).with_session(session).order_by(
        Child.date_of_birth.asc(),
        Child.date_of_registration.asc()
    )
    _children = []
    for _child in _children_from_database:
        _children.append({
            'firstname': _child[0],
            'lastname': _child[1],
            'date_of_registration': rearrange_date.to_list(_child[2])[0],
            'date_of_birth': rearrange_date.to_list(_child[3])[0],
            'id': _child[4],
            '_starts_in': calculate_startyear(_child[3])
        })
    return render_template('index_children.html',
                           Child=_children,
                           _PageTitle='Kinder overzicht')


@children_bp.route('/details_child/<kid>')
def details_child(kid):
    _child_from_database = find_child(kid)
    _family = provision_edit_family(_child_from_database['family_id'])
    _child = []
    if _child_from_database['class_id'] is None:
        _class_name = None
        _class_teacher = None
    else:
        _class_name = find_class(_child_from_database['class_id'])
        _class_teacher = find_teacher(_class_name[1])
    _child.append({
        'firstname': _child_from_database['firstname'],
        'lastname': _child_from_database['lastname'],
        'date_of_registration': rearrange_date.to_list(_child_from_database['date_of_registration'])[0],
        'date_of_birth': rearrange_date.to_list(_child_from_database['date_of_birth'])[0],
        'id': _child_from_database['id'],
        'class_name': _class_name,
        'class_teacher': _class_teacher
    })
    _child_age = calculate_age(_child_from_database['date_of_birth'])
    return render_template('details_child.html',
                           Child=_child,
                           ChildAge=_child_age,
                           Parents=_family,
                           _PageTitle='Kind details')


@children_bp.route('/add_child/', methods=['GET', 'POST'])
def add_child():
    _form1 = AddChild()
    if _form1.validate_on_submit():
        id_field = request.form['id_field']
        firstname = request.form['firstname']
        lastname = request.form['lastname']
        date_of_registration = request.form['date_of_registration']
        date_of_registration = rearrange_date.to_order(date_of_registration)[0]
        family_id = request.form['family_id']
        date_of_birth = request.form['date_of_birth']
        date_of_birth = rearrange_date.to_order(date_of_birth)[0]
        redo_school_year = None
        class_id = None
        # the data to be inserted into Sock model - the table, socks
        record = Child(id_field, firstname, lastname, date_of_registration, family_id, date_of_birth, redo_school_year, class_id)
        # Flask-SQLAlchemy magic adds record to database
        session.add(record)
        session.commit()
        # create a message to send to the template
        _message = f"The data for sock {firstname} {lastname} has been submitted."
        return render_template('add_child.html',
                               message=_message,
                               _PageTitle='Kind toevoegen')
    else:
        # show validaton errors
        # see https://pythonprogramming.net/flash-flask-tutorial/
        for field, errors in _form1.errors.items():
            for error in errors:
                flash("Error in {}: {}".format(
                    getattr(_form1, field).label.text,
                    error
                ), 'error')
        return render_template('add_child.html',
                               form1=_form1,
                               _PageTitle='Kind toevoegen')


@children_bp.route('/edit_child/<cid>')
#TODO
# Klas toewijzen?
def edit_child(cid):
    _id = cid
    _child_from_database = Query([
        Child.firstname,
        Child.lastname,
        Child.date_of_birth,
        Child.date_of_registration,
        Child.id,
        Child.family_id,
        Child.class_id
    ]).with_session(session).filter(Child.id == _id).first()
    # two forms in this template
    _child_to_edit = {'firstname': _child_from_database.firstname,
                      'lastname': _child_from_database.lastname,
                      'date_of_birth': rearrange_date.to_list(_child_from_database.date_of_birth)[0],
                      'date_of_registration': rearrange_date.to_list(_child_from_database.date_of_registration)[0],
                      'id': _child_from_database.id,
                      'family_id': _child_from_database.family_id,
                      'class_id': _child_from_database.class_id}
    # available_groups = query_class_by_year(calculate_startyear(Query(Child.date_of_birth).filter_by(id=cid).with_session(session)[0][0])[3:7])
    # #Now forming the list of tuples for SelectField
    # groups_list = []
    # for i in available_groups:
    #     groups_list.append((i["class_id"], i["class_name"]))
    _form1 = EditChild()
    _form1.class_id.choices = get_appropriate_class(cid)
    return render_template('edit_child.html',
                           childToEdit=_child_to_edit,
                           form1=_form1,
                           _PageTitle='Kind wijzigen')


@children_bp.route('/remove_child/<cid>')
def remove_child(cid):
    _child_to_remove = Query(Child).with_session(session).filter(Child.id == cid).delete()
    session.commit()
    message = f"De gegevens zijn verwijderd."
    return render_template('remove_child_result.html',
                           message=message,
                           _PageTitle='Kind verwijderen')


@children_bp.route('/edit_child_result', methods=['POST'])
def edit_child_result():
    cid = request.form['id_field']
    _child_parents = Query(Child.family_id).filter(Child.id == cid).with_session(session).first()[0]
    _childToEdit = {"id": cid,
                    "firstname": request.form['firstname'],
                    "lastname": request.form['lastname'],
                    "date_of_birth": rearrange_date.to_order(request.form['date_of_birth'])[0],
                    "date_of_registration": rearrange_date.to_order(request.form['date_of_registration'])[0],
                    "family_id": _child_parents,
                    "class_id": request.form['class_id'],
                    "updated": stringdate()}

    available_groups = query_class_by_year(calculate_startyear(Query(Child.date_of_birth).filter_by(id=cid).with_session(session)[0][0])[3:7])
    #Now forming the list of tuples for SelectField
    groups_list = []
    for i in available_groups:
        groups_list.append((i["class_id"], i["class_name"]))
    _form1 = EditChild()
    _form1.class_id.choices = groups_list
    if _form1.validate_on_submit():
        firstname = _form1.firstname.data
        lastname = _form1.lastname.data
        date_of_birth = _form1.date_of_birth.data
        date_of_registration = _form1.date_of_registration.data
        class_id = _form1.class_id.data
        _child_to_edit = Query(Child).with_session(session).filter(Child.id == cid).update(dict(
            firstname=firstname,
            lastname=lastname,
            date_of_registration=date_of_registration,
            date_of_birth=date_of_birth,
            class_id=class_id))
        session.commit()
        message = f"De gegevens voor {_childToEdit['firstname']} zijn bijgewerkt."
        return render_template('result.html',
                               message=message,
                               redirect=f'details_child/{cid}',
                               _PageTitle='Resultaat')
    else:
        # show validaton errors
        _childToEdit["id"] = cid
        # see https://pythonprogramming.net/flash-flask-tutorial/
        for field, errors in _form1.errors.items():
            for error in errors:
                flash("Error in {}: {}".format(
                    getattr(_form1, field).label.text,
                    error
                ), 'error')
        return render_template('edit_child.html',
                               form1=_form1,
                               childToEdit=_childToEdit,
                               choice='edit',
                               redirect=f'details_child/{cid}',
                               _PageTitle='Resultaat')




from flask import render_template, Blueprint, request, flash
from sqlalchemy.orm import Query

from src.classes.base import RearrangeDate
from src.classes.family import AddFamily, EditFamily, find_children
from src.classes.database import Child, Family, sessionSetup, Parent
from src.classes.parent import EditParent, AddParent
from src.modules.common import stringdate

session = sessionSetup()
rearrange_date = RearrangeDate()


# Blueprint Configuration
parents_bp = Blueprint(
    'parents_bp', __name__,
    template_folder='templates'
)


@parents_bp.route('/index_parents')
def index_parents():
    # get a list of unique values in the style column
    _parents_from_database = Query([
        Parent.id,
        Parent.lastname,
        Parent.firstname,
        Parent.address,
        Parent.zipcode,
        Parent.city,
        Parent.email,
        Parent.phone
    ]).with_session(session).order_by(
        Parent.lastname.asc(),
    )
    _parents = []
    for _parent in _parents_from_database:
        _parents.append({
            'id': _parent[0],
            'lastname': _parent[1],
            'firstname': _parent[2],
            'address': _parent[3],
            'zipcode': _parent[4],
            'city': _parent[5],
            'email': _parent[6],
            'phone': _parent[7]
        })
    return render_template('index_parents.html',
                           Parents=_parents,
                           _PageTitle='Ouder Overzicht')


@parents_bp.route('/details_parent/<pid>')
def details_parent(pid):
    try:
        _parent_from_database = Query([
            Parent.firstname,
            Parent.lastname,
            Parent.address,
            Parent.zipcode,
            Parent.city,
            Parent.email,
            Parent.phone,
        ]).filter_by(id=pid).with_session(session).first()
        _parent = {
                'id': pid,
                'firstname': _parent_from_database.firstname,
                'lastname': _parent_from_database.lastname,
                'address': _parent_from_database.address,
                'zipcode': _parent_from_database.zipcode,
                'city': _parent_from_database.city,
                'email': _parent_from_database.email,
                'phone': _parent_from_database.phone
            }
        return render_template('details_parent.html',
                               Parent=_parent,
                               _PageTitle='Ouder details')
    except Exception as e:
        # e holds description of the error
        error_text = "<p>The error:<br>" + str(e) + "</p>"
        hed = '<h1>Something is broken.</h1>'
        return hed + error_text


@parents_bp.route('/edit_parent/<pid>')
def edit_parent(pid):
    _parent_from_database = Query([
        Parent.firstname,
        Parent.lastname,
        Parent.address,
        Parent.zipcode,
        Parent.city,
        Parent.email,
        Parent.phone,
    ]).filter_by(id=pid).with_session(session).first()
    _parentToEdit = {
            'id': pid,
            'firstname': _parent_from_database.firstname,
            'lastname': _parent_from_database.lastname,
            'address': _parent_from_database.address,
            'zipcode': _parent_from_database.zipcode,
            'city': _parent_from_database.city,
            'email': _parent_from_database.email,
            'phone': _parent_from_database.phone
        }
    return render_template('edit_parent.html',
                           parentToEdit=_parentToEdit,
                           form1=EditParent(),
                           _PageTitle='Ouder wijzigen')


@parents_bp.route('/add_parent', methods=['GET', 'POST'])
def add_parent():
    _form1 = AddParent()
    if _form1.validate_on_submit():
        id_field = request.form['id_field']
        firstname = request.form['firstname']
        lastname = request.form['lastname']
        address = request.form['address']
        zipcode = request.form['zipcode']
        city = request.form['city']
        email = request.form['email']
        phone = request.form['phone']
        # the data to be inserted into Sock model - the table, socks
        record = Parent(id_field, firstname, lastname, address, zipcode, city, email, phone)
        # Flask-SQLAlchemy magic adds record to database
        session.add(record)
        session.commit()
        # create a message to send to the template
        _message = f"Ouder {firstname} {lastname} is aangemaakt."
        return render_template('add_parent.html',
                               message=_message,
                               _PageTitle='Ouder toevoegen')
    else:
        # show validaton errors
        # see https://pythonprogramming.net/flash-flask-tutorial/
        for field, errors in _form1.errors.items():
            for error in errors:
                flash("Error in {}: {}".format(
                    getattr(_form1, field).label.text,
                    error
                ), 'error')
        return render_template('add_parent.html',
                               form1=_form1,
                               _PageTitle='Ouder toevoegen')



@parents_bp.route('/remove_parent/<pid>')
def remove_parent(pid):
    _parent_to_delete = Query(Parent).with_session(session).filter(Parent.id == pid).delete()
    session.commit()
    message = f"De gegevens zijn verwijderd."
    return render_template('remove_parent_result.html',
                           message=message,
                           _PageTitle='Ouder verwijderen')


@parents_bp.route('/edit_parent_result', methods=['POST'])
def edit_parent_result():
    pid = request.form['id_field']
    _parentToEdit = {"id": pid,
                    "lastname": request.form['lastname'],
                    "firstname": request.form['firstname'],
                    "address": request.form['address'],
                    "zipcode": request.form['zipcode'],
                    "city": request.form['city'],
                    "email": request.form['email'],
                    "phone": request.form['phone'],
                    "updated": stringdate()}
    _form1 = EditParent()
    if _form1.validate_on_submit():
        lastname = _form1.lastname.data
        firstname = _form1.firstname.data
        address = _form1.address.data
        zipcode = _form1.zipcode.data
        city = _form1.city.data
        email = _form1.email.data
        phone = _form1.phone.data
        _parent_to_edit = Query(Parent).with_session(session).filter(Parent.id == pid).update(dict(
            lastname=lastname,
            firstname=firstname,
            address=address,
            zipcode=zipcode,
            city=city,
            email=email,
            phone=phone))
        session.commit()
        message = f"De gegevens voor {_parentToEdit['lastname']} zijn bijgewerkt."
        return render_template('result.html',
                               message=message,
                               redirect=f'details_parent/{pid}',
                               _PageTitle='Resultaat')
    else:
        # show validaton errors
        _parentToEdit["id"] = pid
        # see https://pythonprogramming.net/flash-flask-tutorial/
        for field, errors in _form1.errors.items():
            for error in errors:
                flash("Error in {}: {}".format(
                    getattr(_form1, field).label.text,
                    error
                ), 'error')
        return render_template('edit_parent.html',
                               form1=_form1,
                               parentToEdit=_parentToEdit,
                               redirect=f'details_parent/{pid}',
                               choice='edit',
                               _PageTitle='Resultaat')



from datetime import date, datetime

from flask_wtf import FlaskForm
from wtforms import SubmitField, HiddenField


class DeleteForm(FlaskForm):
    id_field = HiddenField()
    purpose = HiddenField()
    submit = SubmitField('Delete This Sock')


class RearrangeDate:
    def to_order(self, date_entry):
        return [datetime.strptime(date_entry, '%d-%m-%Y').strftime('%Y-%m-%d') for d in date_entry]

    def to_list(self, date_entry):
        return [datetime.strptime(date_entry, '%Y-%m-%d').strftime('%d-%m-%Y') for d in date_entry]




from datetime import date

from sqlalchemy.orm import Query

from src.classes.database import Child, sessionSetup, Teacher, Class, Family, Parent

session = sessionSetup()


def calculate_age(geboortedatum):
    from datetime import datetime, date

    _date_str = geboortedatum
    _dto = datetime.strptime(_date_str, '%Y-%m-%d').date()
    _today = date.today()
    calculated_age = _today.year - _dto.year - ((_today.month, _today.day) < (_dto.month, _dto.day))
    # self._leeftijd = age
    return calculated_age


def query_class_by_year(child_start_date):
    classes_from_db = Query(Class).with_entities(Class.id, Class.start_date, Class.class_name, Class.teacher, Class.end_date).with_session(session).order_by(Class.start_date.asc())
    classes = []
    for c in classes_from_db:
        if c[1][0:4] <= child_start_date <= c[4][0:4]:
            _c_teacher = Query(Teacher).with_entities(Teacher.id, Teacher.firstname, Teacher.lastname).with_session(session).filter_by(id=c[3])[0]
            classes.append({
                'class_id': c[0],
                'class_name': f"{c[2]} {c[1][0:4]} - {_c_teacher[1]} {_c_teacher[2]}"
            })
    return classes


def calculate_startyear(date_of_birth):
    from datetime import datetime
    from dateutil.relativedelta import relativedelta

    datetime_object = datetime.strptime(date_of_birth, "%Y-%m-%d").date()
    new_date = datetime_object + relativedelta(years=4)
    return datetime.strftime(new_date, "%m-%Y").replace(' 0', ' ')


def find_class(cid):
    return Query(Class).with_entities(
        Class.class_name,
        Class.teacher,
        Class.start_date,
        Class.end_date,
        Class.id
    ).filter(Class.id == cid).with_session(session)[0]


def find_parents(pid):
    return Query(Parent).with_entities(
        Parent.id,
        Parent.firstname,
        Parent.lastname,
        Parent.address,
        Parent.zipcode,
        Parent.city,
        Parent.phone,
        Parent.email
    ).filter(Parent.id == pid).with_session(session).first()


def find_teacher(tid):
    return Query(Teacher).with_entities(
        Teacher.firstname,
        Teacher.lastname,
        Teacher.id
    ).filter(Teacher.id == tid).with_session(session)[0]


def find_child(kid):
    return Query(Child).with_entities(
        Child.firstname,
        Child.lastname,
        Child.date_of_registration,
        Child.date_of_birth,
        Child.id,
        Child.family_id,
        Child.class_id
    ).filter_by(id=kid).with_session(session).first()


def find_waiting_children():
    _children_from_database = Query(Child).with_entities(
        Child.id,
        Child.firstname,
        Child.lastname,
        Child.date_of_birth,
        Child.date_of_registration,
        Child.class_id
    ).filter(Child.class_id == None).order_by(
        Child.date_of_birth.asc(),
        Child.date_of_registration.asc()
    ).with_session(session).all()

    _w_children = []
    for c in _children_from_database:
        _starts_in_year = calculate_startyear(c[3])
        _w_children.append({
            'id': c[0],
            'firstname': c[1],
            'lastname': c[2],
            'date_of_birth': c[3],
            'class_id': c[4],
            '_starts_in': _starts_in_year
        })
    return _w_children


def stringdate():
    _today = date.today()
    _date_list = str(_today).split('-')
    # build string in format 01-01-2000
    _date_string = _date_list[1] + "-" + _date_list[2] + "-" + _date_list[0]
    return _date_string


def get_family():
    _families = Query([Family.id, Family.parent1_id, Family.parent2_id]).with_session(session).all()
    _fam_with_parents = []
    for _fam in _families:
        parent1 = find_parents(_fam['parent1_id'])
        parent2 = find_parents(_fam['parent2_id'])
        _fam_with_parents.append({
            'family_id': _fam['id'],
            'parent1': parent1['firstname']+' '+parent1['lastname'],
            'parent2': parent2['firstname']+' '+parent2['lastname']
        })
    return _fam_with_parents


def get_appropriate_class(cid):
    available_groups = query_class_by_year(calculate_startyear(Query(Child.date_of_birth).filter_by(id=cid).with_session(session)[0][0])[3:7])
    groups_list = []
    for i in available_groups:
        groups_list.append((i["class_id"], i["class_name"]))
    return groups_list


def assign_child_to_class():
    pass
#TODO
# vanuit index direct een klas toewijzen, via een popup?


from flask_wtf import FlaskForm
from sqlalchemy.orm import Query
from wtforms import SubmitField, HiddenField, StringField, BooleanField
from wtforms.validators import InputRequired, Length, Regexp
from flask_sqlalchemy import SQLAlchemy
from src.modules.common import calculate_age
from src.classes.database import Child, sessionSetup, Class

db = SQLAlchemy()
session = sessionSetup()


class AddParent(FlaskForm):
    # id used only by update/edit
    id_field = HiddenField()
    firstname = StringField('Voornaam', [InputRequired(),
                                         Regexp(r'^[A-Za-z\s\-\']+$', message="Ongeldige naam invoer"),
                                         Length(min=2, max=25, message="Invoer ongeldig, lengte te kort")
                                         ])
    lastname = StringField('Achternaam', [InputRequired(),
                                          Regexp(r'^[A-Za-z\s\-\']+$', message="Ongeldige naam invoer"),
                                          Length(min=2, max=25, message="Invoer ongeldig, lengte te kort")
                                          ])
    address = StringField('Adres', [
        Length(min=0, max=25, message="Invoer ongeldig, lengte te kort")
    ])
    zipcode = StringField('Postcode', [
        Length(min=0, max=25, message="Invoer ongeldig, lengte te kort")
    ])
    city = StringField('Woonplaats')
    email = StringField('E-mail')
    phone = StringField('Telefoonnummer')
    # updated - date - handled in the route
    updated = HiddenField()
    submit = SubmitField('Add/Update Record')


class EditParent(FlaskForm):
    # id used only by update/edit

    id_field = HiddenField()
    firstname = StringField('Voornaam', [InputRequired(),
                                         Regexp(r'^[A-Za-z\s\-\']+$', message="Ongeldige naam invoer"),
                                         Length(min=2, max=25, message="Invoer ongeldig, lengte te kort")
                                         ])
    lastname = StringField('Achternaam', [InputRequired(),
                                          Regexp(r'^[A-Za-z\s\-\']+$', message="Ongeldige naam invoer"),
                                          Length(min=2, max=25, message="Invoer ongeldig, lengte te kort")
                                          ])
    address = StringField('Adres', [
        Length(min=0, max=25, message="Invoer ongeldig, lengte te kort")
    ])
    zipcode = StringField('Postcode', [
        Length(min=0, max=25, message="Invoer ongeldig, lengte te kort")
    ])
    city = StringField('Woonplaats')
    email = StringField('E-mail')
    phone = StringField('Telefoonnummer')
    updated = HiddenField()
    submit = SubmitField('opslaan')


def find_children(fid):
    _children_from_database = Query(
        [Child.id, Child.firstname, Child.date_of_birth, Child.parents, Child.class_id]).filter(
        Child.parents == fid).with_session(session)
    _child = []
    for _c in _children_from_database:
        if _c[4] is None:
            _class_name = 'N/A'
        else:
            _class_name = Query(Class.class_name).filter(Class.id == _c[4]).with_session(session)[0][0]
        _child.append({
            'id': _c[0],
            'firstname': _c[1],
            'date_of_birth': _c[2],
            'child_age': calculate_age(_c[2]),
            'parents': _c[3],
            'class': _class_name
        })
    return _child



from flask_wtf import FlaskForm
from sqlalchemy.orm import Query
from wtforms import SubmitField, SelectField, HiddenField, StringField, DateField, BooleanField
from wtforms.validators import InputRequired, Length, Regexp

from src.classes.base import RearrangeDate
from src.classes.database import Family, sessionSetup, Child
from src.modules.common import find_parents, get_family, get_appropriate_class
from src.modules.families import find_families

session = sessionSetup()
rearrange_date = RearrangeDate()


class AddChild(FlaskForm):
    # id used only by update/edit
    id_field = HiddenField()
    firstname = StringField('Voornaam', [InputRequired(),
                                         Regexp(r'^[A-Za-z\s\-\']+$', message="Invalid sock name"),
                                         Length(min=2, max=25, message="Invalid sock name length")
                                         ])
    lastname = StringField('Achternaam', [InputRequired(),
                                          Regexp(r'^[A-Za-z\s\-\']+$', message="Invalid sock name"),
                                          Length(min=2, max=25, message="Invalid sock name length")
                                          ])
    date_of_registration = DateField('Datum van inschrijving', [InputRequired()],
                                     format='%d-%m-%Y',
                                     render_kw={"placeholder": "dd-mm-jjjj"})
    family_id = SelectField(u'Ouders', [InputRequired()],
                         coerce=int,
                         choices=[(_fam['family_id'], _fam['parent1']+' & '+_fam['parent2']) for _fam in get_family()])
    date_of_birth = DateField("Geboortedatum", [InputRequired()], format='%d-%m-%Y',
                              render_kw={"placeholder": "dd-mm-jjjj"})

    # updated - date - handled in the route
    updated = HiddenField()
    submit = SubmitField('opslaan')


class EditChild(FlaskForm):
    # id used only by update/edit
    id_field = HiddenField()
    firstname = StringField('Voornaam', [InputRequired(),
                                         Regexp(r'^[A-Za-z\s\-\']+$', message="Invalid sock name"),
                                         Length(min=2, max=25, message="Invalid sock name length")
                                         ])
    lastname = StringField('Achternaam', [InputRequired(),
                                          Regexp(r'^[A-Za-z\s\-\']+$', message="Invalid sock name"),
                                          Length(min=2, max=25, message="Invalid sock name length")
                                          ])
    date_of_registration = DateField('Datum van inschrijving', [InputRequired()], format='%d-%m-%Y',
                                     render_kw={"placeholder": "dd-mm-jjjj"})
    family_id = SelectField(u'Ouders', [InputRequired()],
                         coerce=int,
                         choices=[(_fam['family_id'], _fam['parent1']+' & '+_fam['parent2']) for _fam in get_family()])
    date_of_birth = DateField("Geboortedatum", [InputRequired()], format='%d-%m-%Y',
                              render_kw={"placeholder": "dd-mm-jjjj"})
    class_id = SelectField(u'Klas', coerce=int,)
                           # choices=[_class.class_name for _class in get_appropriate_class(Child.id)])
    redo_school_year = BooleanField(u'Schooljaar overdoen')
    # updated - date - handled in the route
    updated = HiddenField()
    submit = SubmitField('opslaan')



from setuptools import setup, find_packages

setup(
    name='werfklas',
    version='0.0.3',
    packages=['cfg',
              'src',
              'src.classes',
              'src.modules',
              'werfklas.static',
              'werfklas.static.js',
              'frontend',
              'frontend.children',
              'frontend.children.templates',
              'frontend.classrooms',
              'frontend.classrooms.templates',
              'frontend.families',
              'frontend.families.templates',
              'frontend.parents',
              'frontend.parents.templates',
              'frontend.teachers',
              'frontend.teachers.templates',
              'frontend.templates',
              'werfklas'],
    install_requires=[
                        "chardet==5.0.0",
                        "Click==8.1.3",
                        "dominate==2.7.0",
                        "Flask==2.2.2",
                        "Flask-Bootstrap4==4.0.2",
                        "Flask-SQLAlchemy==2.5.1",
                        "itsdangerous==2.1.2",
                        "Jinja2==3.1.2",
                        "MarkupSafe==2.1.1",
                        "requests==2.28.1",
                        "SQLAlchemy==1.4.41",
                        "urllib3==1.26.12",
                        "visitor==0.1.3",
                        "Werkzeug==2.2.2",
                        "Flask-WTF==1.0.1",
                        "WTForms==2.2.1",
                        "python-dateutil",
                        "flask-assets",
                        "PyYAML~=6.0",
                        "pathlib"
    ],
    include_package_data=True,
    url='',
    license='',
    author='Yves van Elk',
    author_email='',
    description=''
)


from sqlalchemy import Column, Integer, String, Boolean
from sqlalchemy import create_engine, ForeignKey
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import backref, relationship, Session
from src.modules.config import load_config

config = load_config()
def sessionSetup():
    engine = create_engine(
        f'sqlite:///{config["database"]["path"] + config["database"]["name"]}' + '?check_same_thread=False',
        echo=False,
    )
    return Session(bind=engine, autoflush=True)


def __init__():
    session = sessionSetup()


Base = declarative_base()


class Audit(Base):
    __tablename__ = 'tbl_audit'
    id = Column(Integer, primary_key=True)
    date = Column(Integer)
    action = Column(String)

    def __init__(self, aid, date, action):
        super().__init__()
        self.aid = aid
        self.date = date
        self.action = action


class Teacher(Base):
    __tablename__ = 'tbl_teachers'
    id = Column(Integer, primary_key=True)
    firstname = Column(String)
    lastname = Column(String)

    def __init__(self, tid, firstname, lastname):
        super().__init__()
        self.tid = tid
        self.firstname = firstname
        self.lastname = lastname


class Class(Base):
    __tablename__ = 'tbl_classrooms'
    id = Column(Integer, primary_key=True)
    class_name = Column(String)
    teacher = Column(Integer, ForeignKey('tbl_teachers.id'))
    teacher_r = relationship("Teacher", backref=backref("tbl_teachers", uselist=False))
    start_date = Column(Integer)
    end_date = Column(Integer)

    def __init__(self, class_id, class_name, teacher, start_date, end_date):
        super().__init__()
        self.class_id = class_id
        self.class_name = class_name
        self.teacher = teacher
        self.start_date = start_date
        self.end_date = end_date


class Parent(Base):
    __tablename__ = 'tbl_parents'
    id = Column(Integer, primary_key=True)
    firstname = Column(String)
    lastname = Column(String)
    address = Column(String)
    zipcode = Column(String)
    city = Column(String)
    email = Column(String)
    phone = Column(String)

    def __init__(self, oid, firstname, lastname, address, zipcode, city, email, phone):
        super().__init__()
        self.oid = oid
        self.firstname = firstname
        self.lastname = lastname
        self.address = address
        self.zipcode = zipcode
        self.city = city
        self.email = email
        self.phone = phone


class Family(Base):
    __tablename__ = 'tbl_families'
    id = Column(Integer, primary_key=True)
    parent1_id = Column(Integer, ForeignKey('tbl_parents.id'))
    parent1 = relationship("Parent", foreign_keys=[parent1_id])
    parent2_id = Column(Integer, ForeignKey('tbl_parents.id'))
    parent2 = relationship("Parent", foreign_keys=[parent2_id])
    divorced = Column(Integer)

    def __init__(self, oid, parent1_id, parent2_id, divorced):
        super().__init__()
        self.oid = oid
        self.parent1_id = parent1_id
        self.parent2_id = parent2_id
        self.divorced = divorced


class Child(Base):
    __tablename__ = 'tbl_children'
    id = Column(Integer, primary_key=True)
    firstname = Column(String)
    lastname = Column(String)
    date_of_registration = Column(Integer)
    family_id = Column(Integer, ForeignKey('tbl_families.id'))
    family = relationship("Family", foreign_keys=[family_id])
    date_of_birth = Column(Integer)
    class_id = Column(Integer, ForeignKey('tbl_classrooms.id'))
    classroom = relationship("Class", foreign_keys=[class_id])
    redo_school_year = Column(Boolean)
    _age = int

    def __init__(self, kid, firstname, lastname, date_of_registration, family_id, date_of_birth, class_id,
                 redo_school_year):
        super().__init__()
        self.kid = kid
        self.firstname = firstname
        self.lastname = lastname
        self.date_of_registration = date_of_registration
        self.family_id = family_id
        self.date_of_birth = date_of_birth
        self.class_id = class_id
        self.redo_school_year = redo_school_year


def create_database(databasefile):
    engine = create_engine('sqlite:///' + databasefile, echo=True)
    Base.metadata.create_all(engine)


if __name__ == '__main__':
    app.run(debug=True)

# def __init__(self):

# ------ lisnters



from src.modules.common import find_class, find_teacher


def get_classroom(cid):
    # provision classroom based upon class id per child.
    # find classroom
    if cid is None:
        return None
    classrooms = find_class(cid)
    # find teacher
    teachers = find_teacher(classrooms['teacher'])
    _classroom = ({
        'name': classrooms['class_name'],
        'id': classrooms['id'],
        'start_date': classrooms['start_date'],
        'end_date': classrooms['end_date'],
        'teacher_id': classrooms['teacher'],
        'teacher_firstname': teachers['firstname'],
        'teacher_lastname': teachers['lastname'],
    })
    return _classroom




import yaml
from pathlib import Path

ROOT_DIR = Path(__file__).parents[2]

def load_config():
    with open(f"{ROOT_DIR}/cfg/config.yml", "r") as ymlfile:
        cfg = yaml.safe_load(ymlfile)
    return cfg
